@@ -16,9 +16,85 @@ from collections import Counter, OrderedDict
 from time import time, sleep
 from string import punctuation
 from praw.handlers import MultiprocessHandler
+from sklearn.feature_extraction import DictVectorizer
+from sklearn.feature_extraction.text import TfidfTransformer
+from sklearn.svm import LinearSVC
+from sklearn.naive_bayes import MultinomialNB
+from sklearn.multiclass import OneVsRestClassifier
 
 
-class RedditWordCounter(object):
+class WordCounter(object):
+    """Performs word counting given an input string.
+
+    Data attributes:
+        stemmer: Porter stemmer used optionally to perform stemming of extracted words
+        stopwords (list): list of stop words used to reject common words such as 'and'
+
+    Methods:
+        tokenize
+        get_word_count
+        remove_punctuation
+        remove_stopwords
+        stem_tokens: perform Porter stemming on a list of words
+    """
+
+    def __init__(self):
+        self.stemmer = PorterStemmer()
+
+        # Load stop-words
+        application_root = os.path.dirname(__file__)
+        stopwords = os.path.join(application_root, 'words/stopwords_english.txt')
+        with open(stopwords, 'rb') as stopwords_file:
+            self.stopwords = [word.strip('\n') for word in stopwords_file.readlines()]
+
+    def tokenize(self, text):
+        """Tokenize an input string into a list of words (with punctuation removed)."""
+        text = text.lower()
+        punctuation_removed = self.remove_punctuation(text)
+        tokens = nltk.word_tokenize(punctuation_removed)
+        return tokens
+
+    def get_word_count(self, text, stop_words=True, stemming=False):
+        """Return a dict (Counter) of words and corresponding counts given an input string."""
+        tokens = self.tokenize(text)
+
+        # Remove stop words
+        if stop_words:
+            tokens = self.remove_stopwords(tokens)
+
+        if stemming:
+            tokens = self.stem_tokens(tokens)
+
+        return Counter(tokens)
+
+    @staticmethod
+    def remove_punctuation(text, replacement=' ', exclude="'"):
+        """Remove punctuation from an input string."""
+        text = text.replace("'", "")  # Single quote always stripped out
+        for p in set(list(punctuation)) - set(list(exclude)):
+            text = text.replace(p, replacement)
+
+        text = ' '.join(text.split())  # Remove excess whitespace
+        return text
+
+    def remove_stopwords(self, tokens):
+        """Remove all stopwords from a list of word tokens."""
+        return [word for word in tokens if word not in self.stopwords]
+
+    def stem_tokens(self, tokens):
+        """Perform porter stemming on a list of word tokens."""
+        return [self.stemmer.stem(word) for word in tokens]
+
+    def count_words_from_list(self, text, word_list, normalize=True):
+        """Count the number of times the words from a given list appear in text."""
+        text = self.tokenize(text)
+        count = sum([1 for word in text if word in word_list])
+        if normalize:
+            count /= len(text)
+        return count
+
+
+class RedditWordCounter(WordCounter):
     """Performs word counting of comments and titles in Reddit using the Reddit API.
 
     To initialise a new RedditWordCounter instance:
@@ -30,17 +106,12 @@ class RedditWordCounter(object):
     Data Attributes:
         user_agent (str): required to connect to Reddit
         reddit: instance of the Reddit API connection
-        stemmer: Porter stemmer used optionally to perform stemming of extracted words
-        stopwords (list): list of stop words used to reject common words such as 'and'
+        word_counter: WordCounter object used to perform word counting given input strings
 
     Methods:
         subreddit_comments: word count from comments of a given subreddit
         subreddit_titles: word count from titles of a given subreddit
         user_comments: word count from comments of a given user
-        get_word_count: return tokenized word counts given an input string
-        remove_punctuation
-        remove_stopwords
-        stem_tokens: perform Porter stemming on a list of words
         check_connection: check that there is a working connection to Reddit
     """
 
@@ -49,16 +120,16 @@ class RedditWordCounter(object):
             user,
             multiprocess=False
     ):
+        """Initialise a RedditWordCounter object.
+
+        :param user: your Reddit username
+        :param multiprocess: if True, will handle requests from multiple RedditWordCounter objects (False by default)
+        :return:
+        """
+        super(RedditWordCounter, self).__init__()  # Initialise the WordCounter class
         handler = MultiprocessHandler() if multiprocess else None
         self.user_agent = 'redditvocab/0.1 bot by {0}'.format(user)
         self.reddit = praw.Reddit(user_agent=self.user_agent, handler=handler)
-        self.stemmer = PorterStemmer()
-
-        # Load stop-words
-        application_root = os.path.dirname(__file__)
-        stopwords = os.path.join(application_root, 'words/stopwords_english.txt')
-        with open(stopwords, 'rb') as stopwords_file:
-            self.stopwords = [word.strip('\n') for word in stopwords_file.readlines()]
 
     def subreddit_comments(self, subreddit_name, limit=1000, stemming=False, get_all_comments=False):
         """Retrieve the vocabulary from the comments of a subreddit.
@@ -98,12 +169,12 @@ class RedditWordCounter(object):
         comments_processed = 0
 
         for submission in subreddit.get_hot(limit=None):
-            comments = praw.helpers.flatten_tree(submission.comments)
+            submission_comments = praw.helpers.flatten_tree(submission.comments)
 
             # Run over all comments
-            submission_vocab, num_new_comments = get_vocabulary(comments)
-            vocabulary += submission_vocab
-            comments_processed += num_new_comments
+            submission_vocabulary, new_comments = get_vocabulary(submission_comments)
+            vocabulary += submission_vocabulary
+            comments_processed += new_comments
 
             print("Comments processed for subreddit '{0}': {1}".format(subreddit_name, comments_processed), end="\r")
 
@@ -171,37 +242,6 @@ class RedditWordCounter(object):
         print('\n')
         return vocabulary
 
-    def get_word_count(self, text, stop_words=True, stemming=False):
-        text = text.lower()
-        punctuation_removed = self.remove_punctuation(text)
-        tokens = nltk.word_tokenize(punctuation_removed)
-
-        # Remove stop words
-        if stop_words:
-            tokens = self.remove_stopwords(tokens)
-
-        if stemming:
-            tokens = self.stem_tokens(tokens)
-
-        return Counter(tokens)
-
-    @staticmethod
-    def remove_punctuation(text, replacement=' ', exclude=""):
-        """Remove punctuation from an input string """
-        for p in set(list(punctuation)) - set(list(exclude)):
-            text = text.replace(p, replacement)
-
-        text = ' '.join(text.split())  # Remove excess whitespace
-        return text
-
-    def remove_stopwords(self, tokens):
-        """Remove all stopwords from a list of word tokens."""
-        return [word for word in tokens if word not in self.stopwords]
-
-    def stem_tokens(self, tokens):
-        """Perform porter stemming on a list of word tokens."""
-        return [self.stemmer.stem(word) for word in tokens]
-
     def check_connection(self, timeout=10):
         """Wait for a server response."""
         header = {'User-Agent': self.user_agent}
@@ -267,7 +307,12 @@ class TfidfCorpus(object):
         get_tfidf
         get_document_tfidfs
         get_top_terms
-        build_feature_array
+        build_feature_matrix
+        train_classifier
+        classify_document
+        count_words_from_list
+        get_mean_word_length
+        check_corpus_path
     """
 
     def __init__(self, corpus_path='corpus.json'):
@@ -279,6 +324,12 @@ class TfidfCorpus(object):
         self.document_lengths = dict()
         self.corpus = dict()
 
+        # Initialise scikit-learn attributes
+        self.vectorizer = None
+        self.tfidf_transformer = None
+        self.feature_matrix = None
+        self.classifier = None
+
         if os.path.isfile(corpus_path):
             self.load()
 
@@ -348,7 +399,7 @@ class TfidfCorpus(object):
         :param document_name: string which uniquely identifies the document
         """
         if document_name in self.document_list:
-            print("Document with name '{0}' already exists in corpus." \
+            print("Document with name '{0}' already exists in corpus."
                   "Do you wish to replace it?".format(document_name))
             while True:
                 replace_doc = raw_input("Response (y/n): ")
@@ -439,23 +490,59 @@ class TfidfCorpus(object):
         sorted_tfidfs = sorted(tfidfs.items(), key=operator.itemgetter(1), reverse=True)
         return OrderedDict(sorted_tfidfs[:num_terms])
 
-    def build_vectorizer(self, tfidf=True):
+    def build_feature_matrix(self, tfidf=True):
         """Transforms the corpus into a scikit-learn vectorizer object which can be used for machine learning.
+        Used to set the object attributes self.vectorizer and self.feature_matrix.
 
-        :param tfidf: if True, applies TfidfTransformer to vectorized features
-        :return: scikit-learn vectorizer
+        :param tfidf (bool): if True, applies TfidfTransformer to vectorized features
+        :return: scikit-learn vectorizer, scipy sparse feature matrix and its corresponding document labels
         """
-        from sklearn.feature_extraction import DictVectorizer
-        from sklearn.feature_extraction.text import TfidfTransformer
 
-        index = [self.get_document(document) for document in self.document_list]
+        train_data = [self.get_document(document) for document in self.document_list]
+        labels = self.document_list
         vectorizer = DictVectorizer()
-        vectorizer.fit_transform(index)
+        feature_matrix = vectorizer.fit_transform(train_data)
 
+        self.tfidf_transformer = None
         if tfidf:
-            vectorizer = TfidfTransformer().fit_transform(vectorizer)
+            self.tfidf_transformer = TfidfTransformer()
+            feature_matrix = self.tfidf_transformer.fit_transform(feature_matrix)
 
-        return vectorizer
+        self.vectorizer = vectorizer
+        self.feature_matrix = feature_matrix
+        return feature_matrix, labels, vectorizer
+
+    def train_classifier(self, classifier_type='LinearSVC', tfidf=True):
+        """Trains a document classifier using the vocabulary and documents contained in the corpus. Uses scikit-learn.
+
+        :param classifier_type (str): 'LinearSVC' or 'MultinomialNB' (LinearSVC by default)
+        :param tfidf (bool): if True, applies TfidfTransformer to vectorized features
+        :return: classifier object
+        """
+        self.build_feature_matrix(tfidf=tfidf)
+
+        if classifier_type.lower() == 'linearsvc':
+            classifier = OneVsRestClassifier(LinearSVC(random_state=0))
+        elif classifier_type.lower() == 'multinomialnb':
+            classifier = OneVsRestClassifier(MultinomialNB())
+        else:
+            raise Exception("Parameter classifier_type only accepts 'MultinomialNB', 'BernoulliNB' or 'LinearSVC'.")
+
+        classifier.fit(self.feature_matrix, self.document_list)
+        self.classifier = classifier
+        return classifier
+
+    def classify_document(self, document):
+        """Classifies an input document using a bag-of-words approach with sparse features.
+
+        :param document (dict): dict or Counter of the form {'word1': freq1, 'word2': freq2, ...}
+        :return (str): label corresponding to the document's classification
+        """
+        test_data = self.vectorizer.transform([document])
+        if self.tfidf_transformer:
+            test_data = self.tfidf_transformer.transform(test_data)
+
+        return self.classifier.predict(test_data)
 
     def count_words_from_list(self, document_name, word_list, normalize=True):
         """Given a list of input words, return the counts of these words in a specified document."""
